"""Script for running a tsunami inundation scenario for Onslow, WA, Australia.

Source data such as elevation and boundary data is assumed to be available in
directories specified by project.py
The output sww file is stored in project.outputtimedir 

The scenario is defined by a triangular mesh created from project.polygon,
the elevation data and a simulated submarine landslide.

Ole Nielsen and Duncan Gray, GA - 2005 and Nick Bartzis, GA - 2006
"""


#-------------------------------------------------------------------------------# Import necessary modules
#-------------------------------------------------------------------------------

# Standard modules
import os 

from os import sep
from os.path import dirname, basename

import time

# Related major packages
from pyvolution.shallow_water import Domain, Reflective_boundary, \
                            Dirichlet_boundary, Time_boundary, File_boundary
from pyvolution.data_manager import convert_dem_from_ascii2netcdf, dem2pts
from pyvolution.combine_pts import combine_rectangular_points_files 
from pyvolution.pmesh2domain import pmesh_to_domain_instance
#from geospatial_data import add_points_files

# Application specific imports
import project                 # Definition of file names and polygons
from smf import slump_tsunami  # Function for submarine mudslide 

from shutil import copy
from os import mkdir, access, F_OK

from geospatial_data import *
import sys
from pyvolution.util import Screen_Catcher

#-------------------------------------------------------------------------------
# Preparation of topographic data
#
# Convert ASC 2 DEM 2 PTS using source data and store result in source data
# Do for coarse and fine data
# Fine pts file to be clipped to area of interest
#-------------------------------------------------------------------------------

# filenames
onshore_dem_name = project.onshore_dem_name
offshore_points1 = project.offshore_dem_name1
offshore_points2 = project.offshore_dem_name2
meshname = project.meshname+'.msh'
source_dir = project.boundarydir




#import sys; sys.exit()

# creates copy of code in output dir if dir doesn't exist
if access(project.outputtimedir,F_OK) == 0 :
    mkdir (project.outputtimedir)
copy (dirname(project.__file__) +sep+ project.__name__+'.py', project.outputtimedir + project.__name__+'.py')
copy (__file__, project.outputtimedir + basename(__file__))


#normal screen output is stored in 
screen_output_name = project.outputtimedir + "screen_output.txt"
screen_error_name = project.outputtimedir + "screen_error.txt"

#used to catch screen output to file
sys.stdout = Screen_Catcher(screen_output_name)
sys.stderr = Screen_Catcher(screen_error_name)


'''
# fine data (clipping the points file to smaller area)
# creates DEM from asc data 
convert_dem_from_ascii2netcdf(onshore_dem_name, use_cache=True, verbose=True)

#creates pts file from DEM
dem2pts(onshore_dem_name,
        easting_min=project.eastingmin,
        easting_max=project.eastingmax,
        northing_min=project.northingmin,
        northing_max= project.northingmax,
        use_cache=True, 
        verbose=True)

print'create G1'
G1 = Geospatial_data(file_name = project.offshore_dem_name1 + '.xya')

print'create G2'
G2 = Geospatial_data(file_name = project.offshore_dem_name2 + '.xya')

print'create G3'
G3 = Geospatial_data(file_name = project.onshore_dem_name + '.pts')

print'add G1+G2+G3'
G = G1 + G2 + G3

print'export G'
G.export_points_file(project.combined_dem_name + '.pts')

'''
#-------------------------------------------------------------------------------                                 
# Create the triangular mesh based on overall clipping polygon with a tagged
# boundary and interior regions defined in project.py along with
# resolutions (maximal area of per triangle) for each polygon
#-------------------------------------------------------------------------------

from pmesh.mesh_interface import create_mesh_from_regions

region_res = 100000
coast_res = 2500
pt_hedland_res = 500
# derive poly_coast from project.coast_name using alpha_shape
interior_regions = [[project.poly_pt_hedland, pt_hedland_res],
                    [project.poly_coast, coast_res]]

print 'number of interior regions', len(interior_regions)

from utilities.polygon import inside_polygon, plot_polygons

bounding_polygon = project.polyAll
count = 0
for i in range(len(interior_regions)):
    region = interior_regions[i]
    interior_polygon = region[0]
    if len(inside_polygon(interior_polygon, bounding_polygon,
                   closed = True, verbose = False)) <> len(interior_polygon):
        print 'WARNING: interior polygon %d is outside bounding polygon' %(i)
        count += 1

#figname = 'pt_hedland_polys'
#plot_polygons([project.polyAll,project.poly_pt_hedland,project.poly_region],
#              figname,
#              verbose = True)

if count == 0:
    print 'interior regions OK'
else:
    print 'check out your interior polygons'
    print 'check %s in production directory' %figname
    import sys; sys.exit()
    
print 'start create mesh from regions'
from caching import cache
_ = cache(create_mesh_from_regions,
          project.polyAll,
#          {'boundary_tags': {'right': [0], 'bottomright': [1],
#                             'bottomleft': [2], 'left': [3], 'top': [4]},
          {'boundary_tags': {'topright': [0], 'top': [1],'topleft': [2], 'left': [3],
                             'bottomleft': [4], 'bottomright': [5], 'right': [6]},
           'maximum_triangle_area': 10000000,
           'filename': meshname,           
           'interior_regions': interior_regions},
          verbose = True)


#-------------------------------------------------------------------------------                                 
# Setup computational domain
#-------------------------------------------------------------------------------                                 
'''
domain = pmesh_to_domain_instance(meshname, Domain,
                                  use_cache = False,
                                  verbose = True)
'''

domain = Domain(meshname, use_cache = False, verbose = True)

print 'Number of triangles = ', len(domain)
print 'The extent is ', domain.get_extent()
print domain.statistics()

domain.set_name(project.basename)
domain.set_datadir(project.outputtimedir)
domain.set_quantities_to_be_stored(['stage', 'xmomentum', 'ymomentum'])

#-------------------------------------------------------------------------------                                 
# Setup initial conditions
#-------------------------------------------------------------------------------

tide = 0.

domain.set_quantity('stage', tide)
domain.set_quantity('friction', 0.0) 
print 'hi and file',project.combined_dem_name + '.pts'

domain.set_quantity('elevation', 
                    filename = project.combined_dem_name + '.pts',
                    use_cache = True,
                    verbose = True,
                    alpha = 0.1
                    )

#print 'have sent quantities OK - now exiting'
#import sys; sys.exit()

#-------------------------------------------------------------------------------                                 
# Setup boundary conditions (all reflective)
#-------------------------------------------------------------------------------
print 'start ferret2sww'
# skipped as results in file SU-AU_clipped is correct for all WA

from pyvolution.data_manager import ferret2sww

south = project.south 
north = project.north
west = project.west
east = project.east

#note only need to do when an SWW file for the MOST boundary doesn't exist
cache(ferret2sww,
      (source_dir + project.boundary_basename,
       source_dir + project.boundary_basename), 
      {'verbose': True,
       'minlat': south,
       'maxlat': north,
       'minlon': west,
       'maxlon': east,
#       'origin': project.mesh_origin,
       'origin': domain.geo_reference.get_origin(),
       'mean_stage': tide,
       'zscale': 1,                 #Enhance tsunami
       'fail_on_NaN': False,
       'inverted_bathymetry': True},
      #evaluate = True,
       verbose = True)


print 'Available boundary tags', domain.get_boundary_tags()

Bf = File_boundary(source_dir + project.boundary_basename + '.sww', 
                    domain, verbose = True)
Br = Reflective_boundary(domain)
Bd = Dirichlet_boundary([tide,0,0])

# 7 min square wave starting at 1 min, 6m high
Bw = Time_boundary(domain = domain,
                   f=lambda t: [(60<t<480)*6, 0, 0])

#domain.set_boundary( {'top': Bf, 'left': Br, 'bottomleft': Br, 'bottomright': Br, 'right': Br} )
#
domain.set_boundary( {'topright': Bf, 'top': Bf,'topleft': Bf, 'left':  Br,
                             'bottomleft': Br, 'bottomright': Br, 'right': Br})
                             
#-------------------------------------------------------------------------------                                 
# Evolve system through time
#-------------------------------------------------------------------------------
import time
t0 = time.time()

for t in domain.evolve(yieldstep = 240, finaltime = 7200): 
    domain.write_time()
    domain.write_boundary_statistics(tags = 'top')     

for t in domain.evolve(yieldstep = 120, finaltime = 12600
                       ,skip_initial_step = True): 
    domain.write_time()
    domain.write_boundary_statistics(tags = 'top')     

for t in domain.evolve(yieldstep = 60, finaltime = 19800
                       ,skip_initial_step = True): 
    domain.write_time()
    domain.write_boundary_statistics(tags = 'top')     
    
for t in domain.evolve(yieldstep = 120, finaltime = 25200
                       ,skip_initial_step = True): 
    domain.write_time()
    domain.write_boundary_statistics(tags = 'top')     

for t in domain.evolve(yieldstep = 240, finaltime = 36000
                       ,skip_initial_step = True): 
    domain.write_time()
    domain.write_boundary_statistics(tags = 'top')     
  
print 'That took %.2f seconds' %(time.time()-t0)

print 'finished'
